// This file describes the API available to user application to control the
// NFSP daemon running locally
//
// Author: NFSP consortium 8)
// Date: 2002/07
// Licence: GPLv2

#ifndef __NFSPDCTL_H__
#define __NFSPDCTL_H__ 1

#define NFSPDCTL_API_VERSION 102

#include <netinet/in.h>		// for struct sockaddr_in

// These functions all follow the same convention for return values:
//  - ret < 0 => indicates an error (errno is set)
//  - ret >= 0 => success and/or expected value
//
// NB: if errno indicates ENOTCONN or ECONNABORTED there's most likely a bad
// connection with the unix socket


// opens the connection with the nfspd
int nfspd_initialize();

// closes the connection with the nfspd
int nfspd_terminate();

// creates a new iod descriptor given its address and port
//
// required:
//  nbviods is >= 0
// 
// NB: returns the iod_handler >= 0 in case of success
int nfspd_addiod(const struct sockaddr_in *addr);

// free an iod descriptor
// required:
//  ih exists
int nfspd_freeiod(const int ih);

// bind an iod to a viod
//
// required:
//  ih exists
//  iviod is in 0..nbviods-1
int nfspd_bindiod(int iviod, int ih);

// unbind an iod to a viod
//
// required:
//  iviod  must be in 0..nbviods-1
//  ih exists and is bound to iviod
int nfspd_unbindiod(int ih);

// set nbviod
//
// required:
//  nb >=0
int nfspd_setnbviods(int nb);

// get the number of viods
//
// NB: returns the number of viods (>=0)
int nfspd_getnbviods();

// ask the nfspd to go in DROP mode
int nfspd_drop();

// ask the nfspd to go in FORWARD mode
int nfspd_forward();

// ask the nfspd to reset its mode (a.k.a. FLUSH operation)
int nfspd_flush();

// ask the nfspd how many iods are in the viod
// 
// return:
//  <0 indicates an error
//   0 indicates no iod left in this viod
//  >0 there is at least 1 iod in this viod
//     NB: it currently always return 1 even though there are more iods
//
// NB: sticking to the 3 cases <0, ==0 and >0 is safer than expecting to have
// the realnumber.
int nfspd_get_viod_size(const int iviod);

#endif

