// To use:
//   ./nfsp_api_test config_file
//   
// To compile:
// 
// gcc -D_GNU_SOURCE=1 -O -Wall nfsp_api_test.c nfsp_api.c Xstat.c nfs_iods.c -o nfsp_api_test
//
// To test:
//  ./tst tst.conf

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <netinet/in.h>

#include "nfspstriping.h"
#define MAXLEN 1024

void safegets(char *buf, int buflen) {
	int len;
	int i;

	fgets(buf, buflen, stdin);
	buf[buflen - 1] = 0;
	len = strlen(buf);

	printf("\n*** len %d buflen %d ***\n", len, buflen);

	for (i = len - 1; i >= 0 ; i--) {
		if (buf[i] == '\r' || buf[i] == '\n') {
			buf[i] = 0;
		}
	}
}

// configfile file offset size
int main(int argc, char *argv[]) {
	int i;
	char filename[MAXLEN + 1];

 	nfsp_dbglog_enable();

	if (argc != 2) {
		printf("Usage: %s configfile\n", argv[0]);
		exit(1);
	}
	if (nfsp_init(argv[1])) {
		printf("Error while parsing nfsp_init\n");
		exit(1);
	}

	printf("\n\nFilename ?\n");
	safegets(filename, MAXLEN);

	do {
		struct Xstat xs;
		char filesizestr[MAXLEN];

		if (-1 == access(filename, F_OK)) {
			/* Creation */
			
			xs.mfstat.st_uid = geteuid();
			xs.mfstat.st_gid = getegid();
			xs.mfstat.st_mode = 0644;

			xs.nstat.magic = XSTAT_MAGIC;
			xs.nstat.seed = random();
			xs.nstat.realsize = 0;

			printf("Looking for [%s]\n", filename);
			if (-1 == nfsp_meta_update(filename, &xs)) {
				printf("error nfsp_meta_update errno=%d", errno);
			}

		} else {
			printf("File already existing\n");
		}

		printf("Set size to ?\n");
		safegets(filesizestr, MAXLEN);

		printf("Filesize will be set to [%d]\n", atoi(filesizestr));
		xs.nstat.realsize = atoi(filesizestr);
		
		printf("Do not forget to handle truncate by yourself\n");

		if (-1 == nfsp_meta_update(filename, &xs)) {
			printf("error nfsp_meta_update errno=%d", errno);
		}

       
	} while (0);


	do {
		char offsetstr[MAXLEN], sizestr[MAXLEN];
		
		struct nfsp_iod_t *iods;
		int iods_size;
		struct nfsp_iov_t *iov;
		int iov_size;

		if (nfsp_get_iods(&iods, &iods_size)) {
			printf("error nfsp_get_iods\n");
			exit(1);
		}

		for (i = 0; i < iods_size; i++) {
			printf("iodno=%d ip=%8.8x port=%d path=[%s]\n",
			   	i, ntohl(iods[i].ip), ntohs(iods[i].port), iods[i].path);
		}

		printf("iods_size = %d\n", iods_size);

		printf("Read/Write at offset X ?\n");
		safegets(offsetstr, MAXLEN);
		printf("Read/Write X bytes ?\n");
		safegets(sizestr, MAXLEN);

		printf("offset=%d size=%d\n", atoi(offsetstr), atoi(sizestr));

		{
			char *tmp;
			struct Xstat xs;
			if (Xlstat(filename, &xs)) {
				printf("cannot Xlstat [%s]\n",filename);
				exit(1);
			};
			if (nfsp_get_localfn_from_xstat(&tmp, &xs)) {
				printf("cannot nfsp_get_localfn_from_xstat() [%s]\n", filename);
				exit(1);
			};
			printf("using nfsp_get_localfn_from_xstat(): filename[%s] => [%s]\n", filename, tmp);
			nfsp_free_localfn(tmp);
		}

		if (nfsp_get_iov_from_filename(&iov, &iov_size,
				filename, atoi(offsetstr), atoi(sizestr))) {
			printf("errno nfsp_get_iov_from_filename\n");
			exit(1);
		}

		printf("iov_size = %d\n", iov_size);

		for (i = 0; i < iov_size; i++) {
			printf("chunk=%d iod=%d fn=%s off=%lld (fn_off=%lld) size=%d\n",
				i, iov[i].iodno, iov[i].localfn, (int64_t) iov[i].wanted_offset, 
				(int64_t) iov[i].mapped_offset, iov[i].size);
	
		}
	} while (0);

	return 0;
}
